import sum.kern.*;

/**
* @author Amon Portmann & Dario Bsch
* @version 1.0
*/
public class Ball
{
    // Objekte
    private Buntstift hatStift;
    Bildschirm kenntBildschirm;
    boolean fast;
    boolean superball;

    // Konstruktor
    /**
     * Erzeugt einen Ball.
     * @param pBildschirm Der Bildschirm muss bergeben werden.
     */
    public Ball(Bildschirm pBildschirm)
    {
        hatStift = new Buntstift();
        superball = false;
        fast = false;
        kenntBildschirm = pBildschirm;
    }

    // Dienste
    /**
     * Versetzt den Ball in den "fast"-Modus, in dem er sich schneller bewegt.
     * @param modus True aktiviert den Modus, False deaktiviert den Modus.
     */
    public void setzeFast(boolean modus){
        if(modus == false) fast = false;
        else fast = true;
    }
    /**
     * Gibt alle eigenen Objekte frei.
     */
    public void gibFrei()
    {
        hatStift.gibFrei();
    }
    /**
     * Zeichnet den Kreis/Ball mit dem Radius von 5 Pixel.
     */
    public void zeichne()
    {
        hatStift.zeichneKreis(5);
    }
    /**
     * Lscht den Ball
     */
    public void loesche()
    {
        hatStift.radiere();
        this.zeichne();
        hatStift.normal();
    }
    /**
     * Prft, ob der Winkel nach unten gerichtet ist.
     * @param Winkel Der Winkel, der geprft werden soll.
     * @return True, wenn der Winkel nach unten gerichtet ist.
     */
    public boolean nachUnten(double Winkel){
        return ((Winkel%360 > 180 && Winkel%360 < 360)||(Winkel%360 < 0 && Winkel%360 > -180));
    }
    /**
     * Prft, ob der Ball im Supermodus ist.
     */
    public boolean supermodus(){
        if(superball) return true;
        else return false;
    }
    /**
     * Setzt den Ball in den Supermodus (alles zerstrend).
     * @param True aktiviert den Modus, False deaktiviert den Modus.
     */
    public void setzeSuperball(boolean modus){
        if(modus == true){
            superball = true;
            hatStift.setzeFarbe(Farbe.ROT);
        }else{
            superball = false;
            hatStift.setzeFarbe(Farbe.SCHWARZ);
        }
    }
    /**
     * Prft, ob der Winkel nach rechts gerichtet ist.
     */
    public boolean nachRechts(double Winkel){
        return (Winkel > 270 || Winkel < 90);
    }
    /**
     * Prft, ob der Winkel nach links gerichtet ist.
     */
    public boolean nachLinks(double Winkel){
        return (Winkel > 90 && Winkel < 270);
    }
    /**
     * Prft, ob der Winkel nach oben gerichtet ist.
     */
    public boolean nachOben(double Winkel){
        return (Winkel > 0 && Winkel < 180);
    }
    /**
     * Gibt die Richtung/Winkel des Balls zurck. Der Winkel ist immer zwischen 0 und 360!
     * @return Richtung
     */
    public double Richtung(){
        return hatStift.winkel();
    }
    /**
     * Prft, ob der Ball am linken Rand ist.
     */
    public boolean amLinkenRand()
    {
        return hatStift.hPosition() < 5;
    }
     /**
     * Prft, ob der Ball am rechten Rand ist.
     */  
    public boolean amRechtenRand()
    {
        return hatStift.hPosition() > kenntBildschirm.breite()-5;
    }
    /**
     * Prft, ob der Ball am oberen Rand ist.
     */
    public boolean amOberenRand()
    {
        return hatStift.vPosition() < 5;
    }
    /**
     * Prft, ob der Ball am unteren Rand ist.
     */
    public boolean amUnterenRand()
    {
        return hatStift.vPosition() > kenntBildschirm.hoehe() - 11;
    }
    /**
     * Gibt die horizontale Position (Mitte) des Balls zurck.
     * @return horizontale Position
     */
    public double hPosition(){
        return hatStift.hPosition();   
    }
     /**
     * Gibt die vertikale Position (Mitte) des Balls zurck.
     * @return vertikale Position
     */
    public double vPosition(){
        return hatStift.vPosition();   
    }
    /**
     * Setzt eine neue Richtung.
     */
    public void setzeRichtung(double pRichtung)
    {
        hatStift.dreheBis(pRichtung);
    }
    /**
     * Setzt den Ball an die angegebene Position.
     */
    public void setzeBall(int hPos, int vPos)
    {
        hatStift.bewegeBis(hPos,vPos);
    }
    /**
     * Bewegt den Ball um 3 Pixel. Diese Methode wird bentigt, um zu verhindern, dass der Ball an der Wand, einem Brick oder am Balken
     * klebt. Sie wird von eben genannten aufgerufen, wenn der Ball sie berhrte, um ihn vom Bereich, wo der Winkel verndert wird, wegzubringen.
     */
    public void move(){
        this.loesche();
        hatStift.bewegeUm(3);
        this.zeichne();
    }
    /**
     * Bewegt den Ball um 1.8, bzw. 2.5 Pixel (Je nach "fast" - Modus). Wenn der Ball am Rand ankommt prallt er im Einfallswinkel zurck.
     */
    public void bewege()
    {
        this.loesche();
        if(fast){
            hatStift.bewegeUm(2.5);
        }else{
            hatStift.bewegeUm(1.8);
        }
        this.zeichne();
        
        if (this.amLinkenRand())
        {
            this.setzeRichtung(hatStift.winkel()*(-1) + 180);
            this.move();
        }
        
        if (this.amRechtenRand())
        {
            this.setzeRichtung(hatStift.winkel()*(-1) + 180);
            this.move();
        }
        
        if (this.amOberenRand())
        {
            this.setzeRichtung(hatStift.winkel()*(-1));
            this.move();
        }
    }
}
